@@ -1,8 +1,9 @@
 package Mojolicious;
-use Mojo::Base 'Mojo';
+use Mojo::Base -base;
 
 # "Fry: Shut up and take my money!"
 use Carp ();
+use Mojo::DynamicMethods -dispatch;
 use Mojo::Exception;
 use Mojo::Home;
 use Mojo::Log;
@@ -19,11 +20,7 @@ use Mojolicious::Types;
 use Mojolicious::Validator;
 use Scalar::Util ();
 
-has commands => sub {
-  my $commands = Mojolicious::Commands->new(app => shift);
-  Scalar::Util::weaken $commands->{app};
-  return $commands;
-};
+has commands         => sub { Mojolicious::Commands->new(app => shift) };
 has controller_class => 'Mojolicious::Controller';
 has home             => sub { Mojo::Home->new->detect(ref shift) };
 has log              => sub {
@@ -37,6 +34,7 @@ has log              => sub {
     if -d $home->child('log') && -w _;
 
   # Reduced log output outside of development mode
+  return $log->level($ENV{MOJO_LOG_LEVEL}) if $ENV{MOJO_LOG_LEVEL};
   return $mode eq 'development' ? $log : $log->level('info');
 };
 has 'max_request_size';
@@ -45,7 +43,7 @@ has moniker  => sub { Mojo::Util::decamelize ref shift };
 has plugins  => sub { Mojolicious::Plugins->new };
 has renderer => sub { Mojolicious::Renderer->new };
 has routes   => sub { Mojolicious::Routes->new };
-has secrets  => sub {
+has secrets => sub {
   my $self = shift;
 
   # Warn developers about insecure default
@@ -54,46 +52,42 @@ has secrets  => sub {
   # Default to moniker
   return [$self->moniker];
 };
-has sessions => sub { Mojolicious::Sessions->new };
-has static   => sub { Mojolicious::Static->new };
-has types    => sub { Mojolicious::Types->new };
-has ua       => sub {
-  my $ua = Mojo::UserAgent->new;
-  Scalar::Util::weaken $ua->server->app(shift)->{app};
-  return $ua;
-};
+has sessions  => sub { Mojolicious::Sessions->new };
+has static    => sub { Mojolicious::Static->new };
+has types     => sub { Mojolicious::Types->new };
+has ua        => sub { Mojo::UserAgent->new };
 has validator => sub { Mojolicious::Validator->new };
 
-our $CODENAME = 'Doughnut';
-our $VERSION  = '7.70';
+our $CODENAME = 'Supervillain';
+our $VERSION  = '8.22';
 
-sub AUTOLOAD {
-  my $self = shift;
+sub BUILD_DYNAMIC {
+  my ($class, $method, $dyn_methods) = @_;
 
-  my ($package, $method) = our $AUTOLOAD =~ /^(.+)::(.+)$/;
-  Carp::croak "Undefined subroutine &${package}::$method called"
-    unless Scalar::Util::blessed $self && $self->isa(__PACKAGE__);
-
-  # Call helper with fresh controller
-  Carp::croak qq{Can't locate object method "$method" via package "$package"}
-    unless my $helper = $self->renderer->get_helper($method);
-  return $self->build_controller->$helper(@_);
+  return sub {
+    my $self    = shift;
+    my $dynamic = $dyn_methods->{$self->renderer}{$method};
+    return $self->build_controller->$dynamic(@_) if $dynamic;
+    my $package = ref $self;
+    Carp::croak qq{Can't locate object method "$method" via package "$package"};
+  };
 }
 
 sub build_controller {
   my ($self, $tx) = @_;
-  $tx ||= $self->build_tx;
 
   # Embedded application
   my $stash = {};
-  if (my $sub = $tx->can('stash')) { ($stash, $tx) = ($tx->$sub, $tx->tx) }
+  if ($tx && (my $sub = $tx->can('stash'))) {
+    ($stash, $tx) = ($tx->$sub, $tx->tx);
+  }
 
   # Build default controller
   my $defaults = $self->defaults;
   @$stash{keys %$defaults} = values %$defaults;
   my $c
     = $self->controller_class->new(app => $self, stash => $stash, tx => $tx);
-  Scalar::Util::weaken $c->{app};
+  $c->{tx} ||= $self->build_tx;
 
   return $c;
 }
@@ -109,6 +103,7 @@ sub build_tx {
   return $tx;
 }
 
+sub config   { Mojo::Util::_stash(config   => @_) }
 sub defaults { Mojo::Util::_stash(defaults => @_) }
 
 sub dispatch {
@@ -123,13 +118,14 @@ sub dispatch {
 
   # Start timer (ignore static files)
   my $stash = $c->stash;
-  unless ($stash->{'mojo.static'} || $stash->{'mojo.started'}) {
+  $self->log->debug(sub {
     my $req    = $c->req;
     my $method = $req->method;
     my $path   = $req->url->path->to_abs_string;
-    $self->log->debug(qq{$method "$path"});
+    my $id     = $req->request_id;
     $c->helpers->timing->begin('mojo.timer');
-  }
+    return qq{$method "$path" ($id)};
+  }) unless $stash->{'mojo.static'};
 
   # Routes
   $plugins->emit_hook(before_routes => $c);
@@ -148,7 +144,6 @@ sub handler {
 
   # Process with chain
   my $c = $self->build_controller(@_);
-  Scalar::Util::weaken $c->{tx};
   $self->plugins->emit_chain(around_dispatch => $c);
 
   # Delayed response
@@ -171,11 +166,10 @@ sub new {
   my $r = $self->routes->namespaces(["@{[ref $self]}::Controller", ref $self]);
 
   # Hide controller attributes/methods
-  $r->hide(qw(app continue cookie every_cookie every_param));
-  $r->hide(qw(every_signed_cookie finish flash helpers match on param));
-  $r->hide(qw(redirect_to render render_later render_maybe render_to_string));
-  $r->hide(qw(rendered req res respond_to send session signed_cookie stash));
-  $r->hide(qw(tx url_for validation write write_chunk));
+  $r->hide(qw(app cookie every_cookie every_param every_signed_cookie finish));
+  $r->hide(qw(helpers match on param render render_later render_maybe));
+  $r->hide(qw(render_to_string rendered req res send session signed_cookie));
+  $r->hide(qw(stash tx url_for write write_chunk));
 
   $self->plugin($_)
     for qw(HeaderCondition DefaultHelpers TagHelpers EPLRenderer EPRenderer);
@@ -193,6 +187,8 @@ sub plugin {
   $self->plugins->register_plugin(shift, $self, @_);
 }
 
+sub server { $_[0]->plugins->emit_hook(before_server_start => @_[1, 0]) }
+
 sub start {
   my $self = shift;
   $_->warmup for $self->static, $self->renderer;
@@ -201,10 +197,11 @@ sub start {
 
 sub startup { }
 
+sub _die { CORE::die ref $_[0] ? $_[0] : Mojo::Exception->new(shift)->trace }
+
 sub _exception {
   my ($next, $c) = @_;
-  local $SIG{__DIE__}
-    = sub { ref $_[0] ? CORE::die $_[0] : Mojo::Exception->throw(shift) };
+  local $SIG{__DIE__} = \&_die;
   $c->helpers->reply->exception($@) unless eval { $next->(); 1 };
 }
 
@@ -252,6 +249,20 @@ Take a look at our excellent documentation in L<Mojolicious::Guides>!
 
 L<Mojolicious> will emit the following hooks in the listed order.
 
+=head2 before_server_start
+
+Emitted right before the application server is started, for web servers that
+support it, which includes all the built-in ones (except for
+L<Mojo::Server::CGI>).
+
+  $app->hook(before_server_start => sub {
+    my ($server, $app) = @_;
+    ...
+  });
+
+Useful for reconfiguring application servers dynamically or collecting server
+diagnostics information. (Passed the server and application objects)
+
 =head2 after_build_tx
 
 Emitted right after the transaction is built and before the HTTP request gets
@@ -265,7 +276,7 @@ parsed.
 This is a very powerful hook and should not be used lightly, it makes some
 rather advanced features such as upload progress bars possible. Note that this
 hook will not work for embedded applications, because only the host application
-gets to build transactions. (Passed the transaction and application object)
+gets to build transactions. (Passed the transaction and application objects)
 
 =head2 around_dispatch
 
@@ -389,8 +400,7 @@ Useful for rewriting outgoing responses and other post-processing tasks.
 
 =head1 ATTRIBUTES
 
-L<Mojolicious> inherits all attributes from L<Mojo> and implements the
-following new ones.
+L<Mojolicious> implements the following attributes.
 
 =head2 commands
 
@@ -429,9 +439,10 @@ which stringifies to the actual path.
   $app    = $app->log(Mojo::Log->new);
 
 The logging layer of your application, defaults to a L<Mojo::Log> object. The
-level will default to C<debug> if the L</mode> is C<development>, or C<info>
-otherwise. All messages will be written to C<STDERR>, or a C<log/$mode.log> file
-if a C<log> directory exists.
+level will default to either the C<MOJO_LOG_LEVEL> environment variable,
+C<debug> if the L</mode> is C<development>, or C<info> otherwise. All messages
+will be written to C<STDERR>, or a C<log/$mode.log> file if a C<log> directory
+exists.
 
   # Log debug message
   $app->log->debug('It works');
@@ -485,6 +496,9 @@ Used to render content, defaults to a L<Mojolicious::Renderer> object. For more
 information about how to generate content see
 L<Mojolicious::Guides::Rendering>.
 
+  # Enable compression
+  $app->renderer->compress(1);
+
   # Add another "templates" directory
   push @{$app->renderer->paths}, '/home/sri/templates';
 
@@ -540,6 +554,9 @@ session data.
   # Change name of cookie used for all sessions
   $app->sessions->cookie_name('mysession');
 
+  # Disable SameSite feature
+  $app->sessions->samesite(undef);
+
 =head2 static
 
   my $static = $app->static;
@@ -591,20 +608,20 @@ Validate values, defaults to a L<Mojolicious::Validator> object.
 
   # Add validation check
   $app->validator->add_check(foo => sub {
-    my ($validation, $name, $value) = @_;
+    my ($v, $name, $value) = @_;
     return $value ne 'foo';
   });
 
   # Add validation filter
   $app->validator->add_filter(quotemeta => sub {
-    my ($validation, $name, $value) = @_;
+    my ($v, $name, $value) = @_;
     return quotemeta $value;
   });
 
 =head1 METHODS
 
-L<Mojolicious> inherits all methods from L<Mojo> and implements the following
-new ones.
+L<Mojolicious> inherits all methods from L<Mojo::Base> and implements the
+following new ones.
 
 =head2 build_controller
 
@@ -623,6 +640,21 @@ Build default controller object with L</"controller_class">.
 
 Build L<Mojo::Transaction::HTTP> object and emit L</"after_build_tx"> hook.
 
+=head2 config
+
+  my $hash = $app->config;
+  my $foo  = $app->config('foo');
+  $app     = $app->config({foo => 'bar', baz => 23});
+  $app     = $app->config(foo => 'bar', baz => 23);
+
+Application configuration.
+
+  # Remove value
+  my $foo = delete $app->config->{foo};
+
+  # Assign multiple values at once
+  $app->config(foo => 'test', bar => 23);
+
 =head2 defaults
 
   my $hash = $app->defaults;
@@ -719,6 +751,12 @@ default exception handling.
 Load a plugin, for a full list of example plugins included in the
 L<Mojolicious> distribution see L<Mojolicious::Plugins/"PLUGINS">.
 
+=head2 server
+
+  $app->server(Mojo::Server->new);
+
+Emits the L</"before_server_start"> hook.
+
 =head2 start
 
   $app->start;
@@ -744,7 +782,7 @@ startup. Meant to be overloaded in a subclass.
     ...
   }
 
-=head1 AUTOLOAD
+=head1 HELPERS
 
 In addition to the L</"ATTRIBUTES"> and L</"METHODS"> above you can also call
 helpers on L<Mojolicious> objects. This includes all helpers from
@@ -766,7 +804,7 @@ that have been bundled for internal use.
 
 =head2 Mojolicious Artwork
 
-  Copyright (C) 2010-2018, Sebastian Riedel.
+  Copyright (C) 2010-2019, Sebastian Riedel.
 
 Licensed under the CC-SA License, Version 4.0
 L<http://creativecommons.org/licenses/by-sa/4.0>.
@@ -789,6 +827,8 @@ L<http://www.apache.org/licenses/LICENSE-2.0>.
 Every major release of L<Mojolicious> has a code name, these are the ones that
 have been used in the past.
 
+8.0, C<Supervillain> (U+1F9B9)
+
 7.0, C<Doughnut> (U+1F369)
 
 6.0, C<Clinking Beer Mugs> (U+1F37B)
@@ -805,12 +845,23 @@ have been used in the past.
 
 =head1 SPONSORS
 
+=over 2
+
+=item
+
+L<Stix|https://stix.no> sponsored the creation of the Mojolicious logo (designed
+by Nicolai Graesdal) and transferred its copyright to Sebastian Riedel.
+
+=item
+
 Some of the work on this distribution has been sponsored by
-L<The Perl Foundation|http://www.perlfoundation.org>, thank you!
+L<The Perl Foundation|http://www.perlfoundation.org>.
+
+=back
 
 =head1 PROJECT FOUNDER
 
-Sebastian Riedel, C<sri@cpan.org>
+Sebastian Riedel, C<kraih@mojolicious.org>
 
 =head1 CORE DEVELOPERS
 
@@ -818,11 +869,17 @@ Current members of the core team in alphabetical order:
 
 =over 2
 
-Jan Henning Thorsen, C<jhthorsen@cpan.org>
+CandyAngel, C<candyangel@mojolicious.org>
 
-Joel Berger, C<jberger@cpan.org>
+Christopher Rasch-Olsen Raa, C<christopher@mojolicious.org>
 
-Marcus Ramberg, C<mramberg@cpan.org>
+Dan Book, C<grinnz@mojolicious.org>
+
+Jan Henning Thorsen, C<batman@mojolicious.org>
+
+Joel Berger, C<jberger@mojolicious.org>
+
+Marcus Ramberg, C<marcus@mojolicious.org>
 
 =back
 
@@ -852,6 +909,8 @@ Alex Efros
 
 Alex Salimon
 
+Alexander Karelas
+
 Alexey Likhatskiy
 
 Anatoly Sharifulin
@@ -860,6 +919,8 @@ Andre Parker
 
 Andre Vieth
 
+Andreas Guldstrand
+
 Andreas Jaekel
 
 Andreas Koenig
@@ -912,12 +973,14 @@ chromatic
 
 Curt Tilmes
 
-Dan Book
-
 Daniel Kimsey
 
+Daniel Mantovani
+
 Danijel Tasov
 
+Dagfinn Ilmari Mannsåker
+
 Danny Thomas
 
 David Davis
@@ -962,6 +1025,8 @@ Ian Goodacre
 
 Ilya Chesnokov
 
+Ilya Rassadin
+
 James Duncan
 
 Jan Jona Javorsek
@@ -1020,6 +1085,8 @@ Michael Gregorowicz
 
 Michael Harris
 
+Michael Jemmeson
+
 Mike Magowan
 
 Mirko Westermeier
@@ -1090,12 +1157,16 @@ Simon Bertrang
 
 Simone Tampieri
 
+Shoichi Kaji
+
 Shu Cho
 
 Skye Shaw
 
 Stanis Trendelenburg
 
+Stefan Adams
+
 Steffen Ullrich
 
 Stephan Kulow
@@ -1144,14 +1215,14 @@ Zoffix Znet
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2008-2018, Sebastian Riedel and others.
+Copyright (C) 2008-2019, Sebastian Riedel and others.
 
 This program is free software, you can redistribute it and/or modify it under
 the terms of the Artistic License version 2.0.
 
 =head1 SEE ALSO
 
-L<https://github.com/kraih/mojo>, L<Mojolicious::Guides>,
-L<http://mojolicious.org>.
+L<https://github.com/mojolicious/mojo>, L<Mojolicious::Guides>,
+L<https://mojolicious.org>.
 
 =cut
