@@ -2,45 +2,42 @@ package Mojolicious::Plugin::TagHelpers;
 use Mojo::Base 'Mojolicious::Plugin';
 
 use Mojo::ByteStream;
-use Mojo::Util 'xml_escape';
+use Mojo::DOM::HTML 'tag_to_html';
 use Scalar::Util 'blessed';
 
 sub register {
   my ($self, $app) = @_;
 
   # Text field variations
-  my @time = qw(date datetime month time week);
+  my @time = qw(date month time week);
   for my $name (@time, qw(color email number range search tel text url)) {
     $app->helper("${name}_field" => sub { _input(@_, type => $name) });
   }
+  $app->helper(datetime_field => sub { _input(@_, type => 'datetime-local') });
 
-  $app->helper(check_box =>
-      sub { _input(shift, shift, value => shift, @_, type => 'checkbox') });
-  $app->helper(csrf_field => \&_csrf_field);
-  $app->helper(file_field =>
-      sub { shift; _tag('input', name => shift, @_, type => 'file') });
+  my @helpers = (
+    qw(csrf_field form_for hidden_field javascript label_for link_to),
+    qw(select_field stylesheet submit_button tag_with_error text_area)
+  );
+  $app->helper($_ => __PACKAGE__->can("_$_")) for @helpers;
 
-  $app->helper(form_for     => \&_form_for);
-  $app->helper(hidden_field => \&_hidden_field);
+  $app->helper(button_to => sub { _button_to(0, @_) });
+  $app->helper(check_box => sub { _input(@_, type => 'checkbox') });
+  $app->helper(csrf_button_to => sub { _button_to(1, @_) });
+  $app->helper(file_field     => sub { _empty_field('file', @_) });
   $app->helper(image => sub { _tag('img', src => shift->url_for(shift), @_) });
-  $app->helper(input_tag => sub { _input(@_) });
-  $app->helper(javascript => \&_javascript);
-  $app->helper(label_for  => \&_label_for);
-  $app->helper(link_to    => \&_link_to);
-
-  $app->helper(password_field => \&_password_field);
-  $app->helper(radio_button =>
-      sub { _input(shift, shift, value => shift, @_, type => 'radio') });
-
-  $app->helper(select_field  => \&_select_field);
-  $app->helper(stylesheet    => \&_stylesheet);
-  $app->helper(submit_button => \&_submit_button);
+  $app->helper(input_tag      => sub { _input(@_) });
+  $app->helper(password_field => sub { _empty_field('password', @_) });
+  $app->helper(radio_button => sub { _input(@_, type => 'radio') });
 
   # "t" is just a shortcut for the "tag" helper
   $app->helper($_ => sub { shift; _tag(@_) }) for qw(t tag);
+}
 
-  $app->helper(tag_with_error => \&_tag_with_error);
-  $app->helper(text_area      => \&_text_area);
+sub _button_to {
+  my ($csrf, $c, $text) = (shift, shift, shift);
+  my $prefix = $csrf ? _csrf_field($c) : '';
+  return _form_for($c, @_, sub { $prefix . _submit_button($c, $text) });
 }
 
 sub _csrf_field {
@@ -48,65 +45,57 @@ sub _csrf_field {
   return _hidden_field($c, csrf_token => $c->helpers->csrf_token, @_);
 }
 
+sub _empty_field {
+  my ($type, $c, $name) = (shift, shift, shift);
+  return _validation($c, $name, 'input', name => $name, @_, type => $type);
+}
+
 sub _form_for {
   my ($c, @url) = (shift, shift);
   push @url, shift if ref $_[0] eq 'HASH';
 
-  # POST detection
-  my @post;
-  if (my $r = $c->app->routes->lookup($url[0])) {
-    my %methods = (GET => 1, POST => 1);
-    do {
-      my @via = @{$r->via || []};
-      %methods = map { $_ => 1 } grep { $methods{$_} } @via if @via;
-    } while $r = $r->parent;
-    @post = (method => 'POST') if $methods{POST} && !$methods{GET};
-  }
+  # Method detection
+  my $r      = $c->app->routes->lookup($url[0]);
+  my $method = $r ? $r->suggested_method : 'GET';
+  my @post   = $method ne 'GET' ? (method => 'POST') : ();
 
-  return _tag('form', action => $c->url_for(@url), @post, @_);
+  my $url = $c->url_for(@url);
+  $url->query({_method => $method}) if @post && $method ne 'POST';
+  return _tag('form', action => $url, @post, @_);
 }
 
 sub _hidden_field {
-  my $c = shift;
-  return _tag('input', name => shift, value => shift, @_, type => 'hidden');
+  my ($c, $name, $value) = (shift, shift, shift);
+  return _tag('input', name => $name, value => $value, @_, type => 'hidden');
 }
 
 sub _input {
   my ($c, $name) = (shift, shift);
   my %attrs = @_ % 2 ? (value => shift, @_) : @_;
 
-  # Special selection value
-  my @values = @{$c->every_param($name)};
-  my $type = $attrs{type} || '';
-  if (@values && $type ne 'submit') {
+  if (my @values = @{$c->every_param($name)}) {
 
     # Checkbox or radiobutton
-    my $value = $attrs{value} // '';
+    my $type = $attrs{type} || '';
     if ($type eq 'checkbox' || $type eq 'radio') {
-      $attrs{value} = $value;
-      $attrs{checked} = 'checked' if grep { $_ eq $value } @values;
+      my $value = $attrs{value} // 'on';
+      delete $attrs{checked};
+      $attrs{checked} = undef if grep { $_ eq $value } @values;
     }
 
     # Others
-    else { $attrs{value} = $values[0] }
+    else { $attrs{value} = $values[-1] }
   }
 
-  return _validation($c, $name, 'input', %attrs, name => $name);
+  return _validation($c, $name, 'input', name => $name, %attrs);
 }
 
 sub _javascript {
   my $c = shift;
-
-  # CDATA
-  my $cb = sub {''};
-  if (ref $_[-1] eq 'CODE' && (my $old = pop)) {
-    $cb = sub { "//<![CDATA[\n" . $old->() . "\n//]]>" }
-  }
-
-  # URL
-  my $src = @_ % 2 ? $c->url_for(shift) : undef;
-
-  return _tag('script', @_, $src ? (src => $src) : (), $cb);
+  my $content
+    = ref $_[-1] eq 'CODE' ? "//<![CDATA[\n" . pop->() . "\n//]]>" : '';
+  my @src = @_ % 2 ? (src => $c->url_for(shift)) : ();
+  return _tag('script', @src, @_, sub {$content});
 }
 
 sub _label_for {
@@ -133,26 +122,19 @@ sub _link_to {
 
 sub _option {
   my ($values, $pair) = @_;
+
   $pair = [$pair => $pair] unless ref $pair eq 'ARRAY';
+  my %attrs = (value => $pair->[1], @$pair[2 .. $#$pair]);
+  delete $attrs{selected} if keys %$values;
+  $attrs{selected} = undef if $values->{$pair->[1]};
 
-  # Attributes
-  my %attrs = (value => $pair->[1]);
-  $attrs{selected} = 'selected' if exists $values->{$pair->[1]};
-  %attrs = (%attrs, @$pair[2 .. $#$pair]);
-
-  return _tag('option', %attrs, sub { xml_escape $pair->[0] });
-}
-
-sub _password_field {
-  my ($c, $name) = (shift, shift);
-  return _validation($c, $name, 'input', @_, name => $name,
-    type => 'password');
+  return _tag('option', %attrs, $pair->[0]);
 }
 
 sub _select_field {
   my ($c, $name, $options, %attrs) = (shift, shift, shift, @_);
 
-  my %values = map { $_ => 1 } @{$c->every_param($name)};
+  my %values = map { $_ => 1 } grep {defined} @{$c->every_param($name)};
 
   my $groups = '';
   for my $group (@$options) {
@@ -168,61 +150,23 @@ sub _select_field {
     else { $groups .= _option(\%values, $group) }
   }
 
-  return _validation($c, $name, 'select', %attrs, name => $name,
-    sub {$groups});
+  return _validation($c, $name, 'select', name => $name, %attrs, sub {$groups});
 }
 
 sub _stylesheet {
   my $c = shift;
-
-  # CDATA
-  my $cb;
-  if (ref $_[-1] eq 'CODE' && (my $old = pop)) {
-    $cb = sub { "/*<![CDATA[*/\n" . $old->() . "\n/*]]>*/" }
-  }
-
-  # "link" or "style" tag
-  my $href = @_ % 2 ? $c->url_for(shift) : undef;
-  return $href
-    ? _tag('link', rel => 'stylesheet', href => $href, @_)
-    : _tag('style', @_, $cb);
+  my $content
+    = ref $_[-1] eq 'CODE' ? "/*<![CDATA[*/\n" . pop->() . "\n/*]]>*/" : '';
+  return _tag('style', @_, sub {$content}) unless @_ % 2;
+  return _tag('link', rel => 'stylesheet', href => $c->url_for(shift), @_);
 }
 
 sub _submit_button {
-  my $c = shift;
-  return _tag('input', value => shift // 'Ok', @_, type => 'submit');
+  my ($c, $value) = (shift, shift // 'Ok');
+  return _tag('input', value => $value, @_, type => 'submit');
 }
 
-sub _tag {
-  my $name = shift;
-
-  # Content
-  my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
-  my $content = @_ % 2 ? pop : undef;
-
-  # Start tag
-  my $tag = "<$name";
-
-  # Attributes
-  my %attrs = @_;
-  if ($attrs{data} && ref $attrs{data} eq 'HASH') {
-    while (my ($key, $value) = each %{$attrs{data}}) {
-      $key =~ y/_/-/;
-      $attrs{lc("data-$key")} = $value;
-    }
-    delete $attrs{data};
-  }
-  $tag .= qq{ $_="} . xml_escape($attrs{$_} // '') . '"' for sort keys %attrs;
-
-  # Empty element
-  unless ($cb || defined $content) { $tag .= ' />' }
-
-  # End tag
-  else { $tag .= '>' . ($cb ? $cb->() : xml_escape($content)) . "</$name>" }
-
-  # Prevent escaping
-  return Mojo::ByteStream->new($tag);
-}
+sub _tag { Mojo::ByteStream->new(tag_to_html(@_)) }
 
 sub _tag_with_error {
   my ($c, $tag) = (shift, shift);
@@ -234,18 +178,16 @@ sub _tag_with_error {
 sub _text_area {
   my ($c, $name) = (shift, shift);
 
-  # Make sure content is wrapped
-  my $cb = ref $_[-1] eq 'CODE' ? pop : sub {''};
+  my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
   my $content = @_ % 2 ? shift : undef;
-  $cb = sub { xml_escape $content }
-    if defined($content = $c->param($name) // $content);
+  $content = $c->param($name) // $content // $cb // '';
 
-  return _validation($c, $name, 'textarea', @_, name => $name, $cb);
+  return _validation($c, $name, 'textarea', name => $name, @_, $content);
 }
 
 sub _validation {
   my ($c, $name) = (shift, shift);
-  return _tag(@_) unless $c->validation->has_error($name);
+  return _tag(@_) unless $c->helpers->validation->has_error($name);
   return $c->helpers->tag_with_error(@_);
 }
 
@@ -260,7 +202,7 @@ Mojolicious::Plugin::TagHelpers - Tag helpers plugin
 =head1 SYNOPSIS
 
   # Mojolicious
-  $self->plugin('TagHelpers');
+  $app->plugin('TagHelpers');
 
   # Mojolicious::Lite
   plugin 'TagHelpers';
@@ -268,10 +210,11 @@ Mojolicious::Plugin::TagHelpers - Tag helpers plugin
 =head1 DESCRIPTION
 
 L<Mojolicious::Plugin::TagHelpers> is a collection of HTML tag helpers for
-L<Mojolicious>.
+L<Mojolicious>, based on the
+L<HTML Living Standard|https://html.spec.whatwg.org>.
 
-Most form helpers can automatically pick up previous input values and will
-show them as default. You can also use
+Most form helpers can automatically pick up previous input values and will show
+them as default. You can also use
 L<Mojolicious::Plugin::DefaultHelpers/"param"> to set them manually and let
 necessary attributes always be generated automatically.
 
@@ -280,11 +223,12 @@ necessary attributes always be generated automatically.
   <%= radio_button country => 'france'  %> France
   <%= radio_button country => 'uk'      %> UK
 
-For fields that failed validation with L<Mojolicious::Controller/"validation">
-the C<field-with-error> class will be automatically added through the
-C<tag_with_error> helper, to make styling with CSS easier.
+For fields that failed validation with
+L<Mojolicious::Plugin::DefaultHelpers/"validation"> the C<field-with-error>
+class will be automatically added through L</"tag_with_error">, to make styling
+with CSS easier.
 
-  <input class="field-with-error" name="age" type="text" value="250" />
+  <input class="field-with-error" name="age" type="text" value="250">
 
 This is a core plugin, that means it is always enabled and its code a good
 example for learning how to build new plugins, you're welcome to fork it.
@@ -296,16 +240,40 @@ by default.
 
 L<Mojolicious::Plugin::TagHelpers> implements the following helpers.
 
+=head2 button_to
+
+  %= button_to Test => 'some_get_route'
+  %= button_to Test => some_get_route => {id => 23} => (class => 'menu')
+  %= button_to Test => 'http://example.com/test' => (class => 'menu')
+  %= button_to Remove => 'some_delete_route'
+
+Generate portable C<form> tag with L</"form_for">, containing a single button.
+
+  <form action="/path/to/get/route">
+    <input type="submit" value="Test">
+  </form>
+  <form action="/path/to/get/route/23" class="menu">
+    <input type="submit" value="Test">
+  </form>
+  <form action="http://example.com/test" class="menu">
+    <input type="submit" value="Test">
+  </form>
+  <form action="/path/to/delete/route?_method=DELETE" method="POST">
+    <input type="submit" value="Remove">
+  </form>
+
 =head2 check_box
 
+  %= check_box 'employed'
   %= check_box employed => 1
-  %= check_box employed => 1, disabled => 'disabled'
+  %= check_box employed => 1, checked => undef, id => 'foo'
 
 Generate C<input> tag of type C<checkbox>. Previous input values will
 automatically get picked up and shown as default.
 
-  <input name="employed" type="checkbox" value="1" />
-  <input disabled="disabled" name="employed" type="checkbox" value="1" />
+  <input name="employed" type="checkbox">
+  <input name="employed" type="checkbox" value="1">
+  <input checked id="foo" name="employed" type="checkbox" value="1">
 
 =head2 color_field
 
@@ -316,9 +284,20 @@ automatically get picked up and shown as default.
 Generate C<input> tag of type C<color>. Previous input values will
 automatically get picked up and shown as default.
 
-  <input name="background" type="color" />
-  <input name="background" type="color" value="#ffffff" />
-  <input id="foo" name="background" type="color" value="#ffffff" />
+  <input name="background" type="color">
+  <input name="background" type="color" value="#ffffff">
+  <input id="foo" name="background" type="color" value="#ffffff">
+
+=head2 csrf_button_to
+
+  %= csrf_button_to Remove => 'some_delete_route'
+
+Same as L</"button_to">, but also includes a L</"csrf_field">.
+
+  <form action="/path/to/delete/route?_method=DELETE" method="POST">
+    <input name="csrf_token" type="hidden" value="fa6a08...">
+    <input type="submit" value="Remove">
+  </form>
 
 =head2 csrf_field
 
@@ -327,7 +306,7 @@ automatically get picked up and shown as default.
 Generate C<input> tag of type C<hidden> with
 L<Mojolicious::Plugin::DefaultHelpers/"csrf_token">.
 
-  <input name="csrf_token" type="hidden" value="fa6a08..." />
+  <input name="csrf_token" type="hidden" value="fa6a08...">
 
 =head2 date_field
 
@@ -335,25 +314,25 @@ L<Mojolicious::Plugin::DefaultHelpers/"csrf_token">.
   %= date_field end => '2012-12-21'
   %= date_field end => '2012-12-21', id => 'foo'
 
-Generate C<input> tag of type C<date>. Previous input values will
-automatically get picked up and shown as default.
+Generate C<input> tag of type C<date>. Previous input values will automatically
+get picked up and shown as default.
 
-  <input name="end" type="date" />
-  <input name="end" type="date" value="2012-12-21" />
-  <input id="foo" name="end" type="date" value="2012-12-21" />
+  <input name="end" type="date">
+  <input name="end" type="date" value="2012-12-21">
+  <input id="foo" name="end" type="date" value="2012-12-21">
 
 =head2 datetime_field
 
   %= datetime_field 'end'
-  %= datetime_field end => '2012-12-21T23:59:59Z'
-  %= datetime_field end => '2012-12-21T23:59:59Z', id => 'foo'
+  %= datetime_field end => '2012-12-21T23:59:59'
+  %= datetime_field end => '2012-12-21T23:59:59', id => 'foo'
 
-Generate C<input> tag of type C<datetime>. Previous input values will
+Generate C<input> tag of type C<datetime-local>. Previous input values will
 automatically get picked up and shown as default.
 
-  <input name="end" type="datetime" />
-  <input name="end" type="datetime" value="2012-12-21T23:59:59Z" />
-  <input id="foo" name="end" type="datetime" value="2012-12-21T23:59:59Z" />
+  <input name="end" type="datetime-local">
+  <input name="end" type="datetime-local" value="2012-12-21T23:59:59">
+  <input id="foo" name="end" type="datetime-local" value="2012-12-21T23:59:59">
 
 =head2 email_field
 
@@ -364,9 +343,9 @@ automatically get picked up and shown as default.
 Generate C<input> tag of type C<email>. Previous input values will
 automatically get picked up and shown as default.
 
-  <input name="notify" type="email" />
-  <input name="notify" type="email" value="nospam@example.com" />
-  <input id="foo" name="notify" type="email" value="nospam@example.com" />
+  <input name="notify" type="email">
+  <input name="notify" type="email" value="nospam@example.com">
+  <input id="foo" name="notify" type="email" value="nospam@example.com">
 
 =head2 file_field
 
@@ -375,8 +354,8 @@ automatically get picked up and shown as default.
 
 Generate C<input> tag of type C<file>.
 
-  <input name="avatar" type="file" />
-  <input id="foo" name="avatar" type="file" />
+  <input name="avatar" type="file">
+  <input id="foo" name="avatar" type="file">
 
 =head2 form_for
 
@@ -396,26 +375,33 @@ Generate C<input> tag of type C<file>.
     %= text_field 'first_name'
     %= submit_button
   % end
+  %= form_for some_delete_route => begin
+    %= submit_button 'Remove'
+  % end
 
 Generate portable C<form> tag with L<Mojolicious::Controller/"url_for">. For
-routes that allow C<POST> but not C<GET>, a C<method> attribute will be
-automatically added.
+routes that do not allow C<GET>, a C<method> attribute with the value C<POST>
+will be automatically added. And for methods other than C<GET> or C<POST>, an
+C<_method> query parameter will be added as well.
 
   <form action="/path/to/login">
-    <input name="first_name" type="text" />
-    <input value="Ok" type="submit" />
+    <input name="first_name" type="text">
+    <input type="submit" value="Ok">
   </form>
   <form action="/path/to/login.txt" method="POST">
-    <input name="first_name" type="text" />
-    <input value="Ok" type="submit" />
+    <input name="first_name" type="text">
+    <input type="submit" value="Ok">
   </form>
   <form action="/path/to/login" enctype="multipart/form-data">
-    <input disabled="disabled" name="first_name" type="text" />
-    <input value="Ok" type="submit" />
+    <input disabled="disabled" name="first_name" type="text">
+    <input type="submit" value="Ok">
   </form>
   <form action="http://example.com/login" method="POST">
-    <input name="first_name" type="text" />
-    <input value="Ok" type="submit" />
+    <input name="first_name" type="text">
+    <input type="submit" value="Ok">
+  </form>
+  <form action="/path/to/delete/route?_method=DELETE" method="POST">
+    <input type="submit" value="Remove">
   </form>
 
 =head2 hidden_field
@@ -425,8 +411,8 @@ automatically added.
 
 Generate C<input> tag of type C<hidden>.
 
-  <input name="foo" type="hidden" value="bar" />
-  <input id="bar" name="foo" type="hidden" value="bar" />
+  <input name="foo" type="hidden" value="bar">
+  <input id="bar" name="foo" type="hidden" value="bar">
 
 =head2 image
 
@@ -435,32 +421,34 @@ Generate C<input> tag of type C<hidden>.
 
 Generate portable C<img> tag.
 
-  <img src="/path/to/images/foo.png" />
-  <img alt="Foo" src="/path/to/images/foo.png" />
+  <img src="/path/to/images/foo.png">
+  <img alt="Foo" src="/path/to/images/foo.png">
 
 =head2 input_tag
 
   %= input_tag 'first_name'
-  %= input_tag first_name => 'Default name'
+  %= input_tag first_name => 'Default'
   %= input_tag 'employed', type => 'checkbox'
 
 Generate C<input> tag. Previous input values will automatically get picked up
 and shown as default.
 
-  <input name="first_name" />
-  <input name="first_name" value="Default name" />
-  <input name="employed" type="checkbox" />
+  <input name="first_name">
+  <input name="first_name" value="Default">
+  <input name="employed" type="checkbox">
 
 =head2 javascript
 
   %= javascript '/script.js'
+  %= javascript '/script.js', defer => undef
   %= javascript begin
     var a = 'b';
   % end
 
 Generate portable C<script> tag for JavaScript asset.
 
-  <script src="/path/to/script.js" />
+  <script src="/path/to/script.js"></script>
+  <script defer src="/path/to/script.js"></script>
   <script><![CDATA[
     var a = 'b';
   ]]></script>
@@ -497,7 +485,7 @@ Generate C<label> tag.
   %= link_to Contact => 'mailto:sri@example.com'
   <%= link_to index => begin %>Home<% end %>
   <%= link_to '/file.txt' => begin %>File<% end %>
-  <%= link_to 'http://mojolicio.us' => begin %>Mojolicious<% end %>
+  <%= link_to 'https://mojolicious.org' => begin %>Mojolicious<% end %>
   <%= link_to url_for->query(foo => 'bar')->to_abs => begin %>Retry<% end %>
 
 Generate portable C<a> tag with L<Mojolicious::Controller/"url_for">, defaults
@@ -511,7 +499,7 @@ to using the capitalized link target as content.
   <a href="mailto:sri@example.com">Contact</a>
   <a href="/path/to/index">Home</a>
   <a href="/path/to/file.txt">File</a>
-  <a href="http://mojolicio.us">Mojolicious</a>
+  <a href="https://mojolicious.org">Mojolicious</a>
   <a href="http://127.0.0.1:3000/current/path?foo=bar">Retry</a>
 
 =head2 month_field
@@ -523,9 +511,9 @@ to using the capitalized link target as content.
 Generate C<input> tag of type C<month>. Previous input values will
 automatically get picked up and shown as default.
 
-  <input name="vacation" type="month" />
-  <input name="vacation" type="month" value="2012-12" />
-  <input id="foo" name="vacation" type="month" value="2012-12" />
+  <input name="vacation" type="month">
+  <input name="vacation" type="month" value="2012-12">
+  <input id="foo" name="vacation" type="month" value="2012-12">
 
 =head2 number_field
 
@@ -536,9 +524,9 @@ automatically get picked up and shown as default.
 Generate C<input> tag of type C<number>. Previous input values will
 automatically get picked up and shown as default.
 
-  <input name="age" type="number" />
-  <input name="age" type="number" value="25" />
-  <input id="foo" max="200" min="0" name="age" type="number" value="25" />
+  <input name="age" type="number">
+  <input name="age" type="number" value="25">
+  <input id="foo" max="200" min="0" name="age" type="number" value="25">
 
 =head2 password_field
 
@@ -547,19 +535,21 @@ automatically get picked up and shown as default.
 
 Generate C<input> tag of type C<password>.
 
-  <input name="pass" type="password" />
-  <input id="foo" name="pass" type="password" />
+  <input name="pass" type="password">
+  <input id="foo" name="pass" type="password">
 
 =head2 radio_button
 
+  %= radio_button 'test'
   %= radio_button country => 'germany'
-  %= radio_button country => 'germany', id => 'foo'
+  %= radio_button country => 'germany', checked => undef, id => 'foo'
 
 Generate C<input> tag of type C<radio>. Previous input values will
 automatically get picked up and shown as default.
 
-  <input name="country" type="radio" value="germany" />
-  <input id="foo" name="country" type="radio" value="germany" />
+  <input name="test" type="radio">
+  <input name="country" type="radio" value="germany">
+  <input checked id="foo" name="country" type="radio" value="germany">
 
 =head2 range_field
 
@@ -570,9 +560,9 @@ automatically get picked up and shown as default.
 Generate C<input> tag of type C<range>. Previous input values will
 automatically get picked up and shown as default.
 
-  <input name="age" type="range" />
-  <input name="age" type="range" value="25" />
-  <input id="foo" max="200" min="200" name="age" type="range" value="25" />
+  <input name="age" type="range">
+  <input name="age" type="range" value="25">
+  <input id="foo" max="200" min="200" name="age" type="range" value="25">
 
 =head2 search_field
 
@@ -583,21 +573,21 @@ automatically get picked up and shown as default.
 Generate C<input> tag of type C<search>. Previous input values will
 automatically get picked up and shown as default.
 
-  <input name="q" type="search" />
-  <input name="q" type="search" value="perl" />
-  <input id="foo" name="q" type="search" value="perl" />
+  <input name="q" type="search">
+  <input name="q" type="search" value="perl">
+  <input id="foo" name="q" type="search" value="perl">
 
 =head2 select_field
 
-  %= select_field country => [qw(de en)]
+  %= select_field country => ['de', 'en']
   %= select_field country => [[Germany => 'de'], 'en'], id => 'eu'
-  %= select_field country => [[Germany => 'de', disabled => 'disabled'], 'en']
+  %= select_field country => [[Germany => 'de', selected => 'selected'], 'en']
   %= select_field country => [c(EU => [[Germany => 'de'], 'en'], id => 'eu')]
-  %= select_field country => [c(EU => [qw(de en)]), c(Asia => [qw(cn jp)])]
+  %= select_field country => [c(EU => ['de', 'en']), c(Asia => ['cn', 'jp'])]
 
 Generate C<select> and C<option> tags from array references and C<optgroup>
-tags from L<Mojo::Collection> objects. Previous input values will
-automatically get picked up and shown as default.
+tags from L<Mojo::Collection> objects. Previous input values will automatically
+get picked up and shown as default.
 
   <select name="country">
     <option value="de">de</option>
@@ -608,7 +598,7 @@ automatically get picked up and shown as default.
     <option value="en">en</option>
   </select>
   <select name="country">
-    <option disabled="disabled" value="de">Germany</option>
+    <option selected="selected" value="de">Germany</option>
     <option value="en">en</option>
   </select>
   <select name="country">
@@ -631,13 +621,15 @@ automatically get picked up and shown as default.
 =head2 stylesheet
 
   %= stylesheet '/foo.css'
+  %= stylesheet '/foo.css', title => 'Foo style'
   %= stylesheet begin
     body {color: #000}
   % end
 
 Generate portable C<style> or C<link> tag for CSS asset.
 
-  <link href="/path/to/foo.css" rel="stylesheet" />
+  <link href="/path/to/foo.css" rel="stylesheet">
+  <link href="/path/to/foo.css" rel="stylesheet" title="Foo style">
   <style><![CDATA[
     body {color: #000}
   ]]></style>
@@ -649,12 +641,12 @@ Generate portable C<style> or C<link> tag for CSS asset.
 
 Generate C<input> tag of type C<submit>.
 
-  <input type="submit" value="Ok" />
-  <input id="foo" type="submit" value="Ok!" />
+  <input type="submit" value="Ok">
+  <input id="foo" type="submit" value="Ok!">
 
 =head2 t
 
-  %=t div => 'test & 123'
+  %= t div => 'test & 123'
 
 Alias for L</"tag">.
 
@@ -662,20 +654,22 @@ Alias for L</"tag">.
 
 =head2 tag
 
+  %= tag 'br'
   %= tag 'div'
-  %= tag 'div', id => 'foo'
-  %= tag div => 'test & 123'
-  %= tag div => (id => 'foo') => 'test & 123'
-  %= tag div => (data => {my_id => 1, Name => 'test'}) => 'test & 123'
+  %= tag 'div', id => 'foo', hidden => undef
+  %= tag 'div', 'test & 123'
+  %= tag 'div', id => 'foo', 'test & 123'
+  %= tag 'div', data => {my_id => 1, Name => 'test'}, 'test & 123'
   %= tag div => begin
     test & 123
   % end
   <%= tag div => (id => 'foo') => begin %>test & 123<% end %>
 
-HTML/XML tag generator.
+Alias for L<Mojo::DOM/"new_tag">.
 
-  <div />
-  <div id="foo" />
+  <br>
+  <div></div>
+  <div id="foo" hidden></div>
   <div>test &amp; 123</div>
   <div id="foo">test &amp; 123</div>
   <div data-my-id="1" data-name="test">test &amp; 123</div>
@@ -686,10 +680,10 @@ HTML/XML tag generator.
 
 Very useful for reuse in more specific tag helpers.
 
-  my $output = $c->tag('div');
-  my $output = $c->tag('div', id => 'foo');
-  my $output = $c->tag(div => '<p>This will be escaped</p>');
-  my $output = $c->tag(div => sub { '<p>This will not be escaped</p>' });
+  my $output = $c->tag('meta');
+  my $output = $c->tag('meta', charset => 'UTF-8');
+  my $output = $c->tag('div', '<p>This will be escaped</p>');
+  my $output = $c->tag('div', sub { '<p>This will not be escaped</p>' });
 
 Results are automatically wrapped in L<Mojo::ByteStream> objects to prevent
 accidental double escaping in C<ep> templates.
@@ -700,7 +694,7 @@ accidental double escaping in C<ep> templates.
 
 Same as L</"tag">, but adds the class C<field-with-error>.
 
-  <input class="foo field-with-error" />
+  <input class="foo field-with-error">
 
 =head2 tel_field
 
@@ -711,41 +705,41 @@ Same as L</"tag">, but adds the class C<field-with-error>.
 Generate C<input> tag of type C<tel>. Previous input values will automatically
 get picked up and shown as default.
 
-  <input name="work" type="tel" />
-  <input name="work" type="tel" value="123456789" />
-  <input id="foo" name="work" type="tel" value="123456789" />
+  <input name="work" type="tel">
+  <input name="work" type="tel" value="123456789">
+  <input id="foo" name="work" type="tel" value="123456789">
 
 =head2 text_area
 
-  %= text_area 'foo'
-  %= text_area 'foo', cols => 40
-  %= text_area foo => 'Default!', cols => 40
-  %= text_area foo => (cols => 40) => begin
-    Default!
+  %= text_area 'story'
+  %= text_area 'story', cols => 40
+  %= text_area story => 'Default', cols => 40
+  %= text_area story => (cols => 40) => begin
+    Default
   % end
 
 Generate C<textarea> tag. Previous input values will automatically get picked
 up and shown as default.
 
-  <textarea name="foo"></textarea>
-  <textarea cols="40" name="foo"></textarea>
-  <textarea cols="40" name="foo">Default!</textarea>
-  <textarea cols="40" name="foo">
-    Default!
+  <textarea name="story"></textarea>
+  <textarea cols="40" name="story"></textarea>
+  <textarea cols="40" name="story">Default</textarea>
+  <textarea cols="40" name="story">
+    Default
   </textarea>
 
 =head2 text_field
 
   %= text_field 'first_name'
-  %= text_field first_name => 'Default name'
-  %= text_field first_name => 'Default name', class => 'user'
+  %= text_field first_name => 'Default'
+  %= text_field first_name => 'Default', class => 'user'
 
-Generate C<input> tag of type C<text>. Previous input values will
-automatically get picked up and shown as default.
+Generate C<input> tag of type C<text>. Previous input values will automatically
+get picked up and shown as default.
 
-  <input name="first_name" type="text" />
-  <input name="first_name" type="text" value="Default name" />
-  <input class="user" name="first_name" type="text" value="Default name" />
+  <input name="first_name" type="text">
+  <input name="first_name" type="text" value="Default">
+  <input class="user" name="first_name" type="text" value="Default">
 
 =head2 time_field
 
@@ -753,25 +747,25 @@ automatically get picked up and shown as default.
   %= time_field start => '23:59:59'
   %= time_field start => '23:59:59', id => 'foo'
 
-Generate C<input> tag of type C<time>. Previous input values will
-automatically get picked up and shown as default.
+Generate C<input> tag of type C<time>. Previous input values will automatically
+get picked up and shown as default.
 
-  <input name="start" type="time" />
-  <input name="start" type="time" value="23:59:59" />
-  <input id="foo" name="start" type="time" value="23:59:59" />
+  <input name="start" type="time">
+  <input name="start" type="time" value="23:59:59">
+  <input id="foo" name="start" type="time" value="23:59:59">
 
 =head2 url_field
 
   %= url_field 'address'
-  %= url_field address => 'http://mojolicio.us'
-  %= url_field address => 'http://mojolicio.us', id => 'foo'
+  %= url_field address => 'https://mojolicious.org'
+  %= url_field address => 'https://mojolicious.org', id => 'foo'
 
 Generate C<input> tag of type C<url>. Previous input values will automatically
 get picked up and shown as default.
 
-  <input name="address" type="url" />
-  <input name="address" type="url" value="http://mojolicio.us" />
-  <input id="foo" name="address" type="url" value="http://mojolicio.us" />
+  <input name="address" type="url">
+  <input name="address" type="url" value="https://mojolicious.org">
+  <input id="foo" name="address" type="url" value="https://mojolicious.org">
 
 =head2 week_field
 
@@ -779,12 +773,12 @@ get picked up and shown as default.
   %= week_field vacation => '2012-W17'
   %= week_field vacation => '2012-W17', id => 'foo'
 
-Generate C<input> tag of type C<week>. Previous input values will
-automatically get picked up and shown as default.
+Generate C<input> tag of type C<week>. Previous input values will automatically
+get picked up and shown as default.
 
-  <input name="vacation" type="week" />
-  <input name="vacation" type="week" value="2012-W17" />
-  <input id="foo" name="vacation" type="week" value="2012-W17" />
+  <input name="vacation" type="week">
+  <input name="vacation" type="week" value="2012-W17">
+  <input id="foo" name="vacation" type="week" value="2012-W17">
 
 =head1 METHODS
 
@@ -799,6 +793,6 @@ Register helpers in L<Mojolicious> application.
 
 =head1 SEE ALSO
 
-L<Mojolicious>, L<Mojolicious::Guides>, L<http://mojolicio.us>.
+L<Mojolicious>, L<Mojolicious::Guides>, L<https://mojolicious.org>.
 
 =cut
